/*
Object class use to represent questions.

Stores the following data:

- question command, sentence, level in the taxonomy, correct answer (all as Strings)
- integer representation of question type
- arrays of type Word for the command, sentence and correct answer
- floating point array to represent weight system
- floating point complexity measure value

Developed by Soham Hanuman Singh, sngsoh004@myuct.ac.za
*/

public class Question implements Comparable<Question>
{
    String command, level, sentence, answer;
    int type;
    Word [] commandWords, sentenceWords, answerWords;
    double [] weights;
    double difficulty;
    
    public Question()
    {
        command="";
        level = "";
        sentence = "";
        answer = "";
        type = 0;
        weights = new double[6];
        difficulty = 0;
    }
    
    public Question(String text) //reads in a question entry from the question pool and splits data accordingly
    {
        String splitted [] = text.split(","); //divide the question entry into tokens (, separated)
        
        type = Integer.parseInt(splitted[0]); //first token is the type
        sentence = splitted[1]; //sentence is the second token
        command = splitted[2]; //command is the third token
        answer = splitted[3]; //answer is the fourth token
        
        String[] commandSplit = command.split(" "); //split the command into individual words and store this in the Word array representation of it
        commandWords = new Word[commandSplit.length];
        for (int i = 0; i < commandSplit.length; i++) {
            commandWords[i] = new Word(commandSplit[i]);
        }
        
        String[] sentenceSplit = sentence.split(" "); //split the sentence into individual words and store this in the Word array representation of it
        sentenceWords = new Word[sentenceSplit.length];
        for (int i = 0; i < sentenceSplit.length; i++) {
            sentenceWords[i] = new Word(sentenceSplit[i]);
        }
        
        String[] answerSplit = answer.split(" "); //split the answer into individual words and store this in the Word array representation of it
        answerWords = new Word[answerSplit.length];
        for (int i = 0; i < answerSplit.length; i++) {
            answerWords[i] = new Word(answerSplit[i]);
        }
       
        weights = new double[6]; //create weight system
        difficulty = 0; //initialize
        level = ""; //initialize
    }
    
    public String getLevel() //determine the level a question falls within Bloom's taxonomy based on weight values
    {
        double max = weights[0];
        int index = 0;
        
        for (int i = 1; i<weights.length; i++)
        {
            if (weights[i]>=max) //simple algorithm to store max weight value and corresponding index
            {
                max = weights[i];
                index = i;
            }
        }
        switch (index) //retrieve and return String representation of level from corresponding index representation 
        {
            case 0:
                return "Knowledge";
            
            case 1:
                return "Comprehension";
                
            case 2:
                return "Application";
            
            case 3:
                return "Analysis";
                
            case 4:
                return "Synthesis";
            
            case 5:
                return "Evaluation";
        }  
        return "Error";
    }
    
    public String getSentence() //return a textual representation of the sentence without annotations
    {
        String temp = "";
        
        for (int i = 0; i < sentenceWords.length; i++) {
            temp+=sentenceWords[i].text+" ";
        }
        return temp;
    }
    
    public String getAnswer() //return a textual representation of the answer without annotations
    {
        String temp = "";
        
        for (int i = 0; i < answerWords.length; i++) {
            temp+=answerWords[i].text+" ";
        }
        return temp;
    }
    
    public String getCommand() //return a textual representation of the command without annotations
    {
        String temp = "";
        
        for (int i = 0; i < commandWords.length; i++) {
            temp+=commandWords[i].text+" ";
        }
        return temp;
    }

    @Override
    public int compareTo(Question o) { //comparator operator
        if (difficulty<o.difficulty) return -1;
        
        else if (difficulty==o.difficulty) return 0;
        
        else return 1;
    }
}