/*

JFrame responsible for presenting the student with the exercise and grading their responses.
Includes the following:

- Question classification
- Complexity measure evaluation
- Exercise composition
- Exericse presentation
- Answer grading

Developed by Soham Hanuman Singh, sngsoh004@myuct.ac.za
*/

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Tester extends javax.swing.JFrame {

    static KeywordList bloomKeywords = new KeywordList(); //all keywords
    static ArrayList<Question> questions = new ArrayList(); //all questions
    public static ArrayList<String> roots = new ArrayList(); //all verb roots
    static int assessmentSize = 0; //assessment size (to be extracted from Configurator)
    public static ArrayList<Question> assessment = new ArrayList(); //all questions to be presented to student
    static ArrayList answered = new ArrayList(); //all answered questions
    static ArrayList chosen = new ArrayList(); //indices of chosen questions
    public static ArrayList answers = new ArrayList(); //student's answers
    public static ArrayList scores = new ArrayList(); //student's scores for each answer
    public static double grade = 0; //students grade
    static int currentQuestion; //index of current question
    static int questionCount; //count of questions answered + current
    static double totalComplexity = 0; //sum of complexities
    
    public Tester() throws FileNotFoundException {
        initComponents();
        
        initialise(); 
        compose();
        
        System.out.println("Questions chosen for the exercise:");
        int i = 0;
        for (Question q : assessment) //printing all the questions with their complexity measure and level in the taxonomy
        {
            i++;
            System.out.println(i+") Type: "+q.type+", "+q.getSentence()+", Difficulty is: "+q.difficulty+", Bloom: "+q.level);
        }
        System.out.println("");
        
        assessGUI();
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        answerField = new javax.swing.JTextField();
        submitButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        descriptionField = new javax.swing.JTextField();
        sentenceField = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setResizable(false);

        submitButton.setText("Submit answer");
        submitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                submitButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Sentence");

        jLabel2.setText("Enter answer here:");

        jLabel3.setText("Instruction");

        descriptionField.setEditable(false);
        descriptionField.setFont(new java.awt.Font("Lucida Grande", 0, 8)); // NOI18N

        sentenceField.setEditable(false);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel3, javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel1)
                            .addComponent(jLabel2))
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addGap(120, 120, 120)
                                .addComponent(submitButton))
                            .addGroup(layout.createSequentialGroup()
                                .addGap(25, 25, 25)
                                .addComponent(answerField))
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addGap(25, 25, 25)
                                .addComponent(sentenceField, javax.swing.GroupLayout.PREFERRED_SIZE, 460, javax.swing.GroupLayout.PREFERRED_SIZE))))
                    .addComponent(descriptionField, javax.swing.GroupLayout.PREFERRED_SIZE, 460, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(52, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(24, 24, 24)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(sentenceField, javax.swing.GroupLayout.PREFERRED_SIZE, 98, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 27, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(descriptionField, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(answerField, javax.swing.GroupLayout.PREFERRED_SIZE, 111, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addGap(18, 18, 18)
                .addComponent(submitButton)
                .addGap(20, 20, 20))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void submitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_submitButtonActionPerformed
        String answer = this.answerField.getText();
        System.out.println("User entered: "+answer);
        answerQuestion(answer);
    }//GEN-LAST:event_submitButtonActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) throws FileNotFoundException {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(Tester.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(Tester.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(Tester.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(Tester.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                try {
                    new Tester().setVisible(true);
                } catch (FileNotFoundException ex) {
                    Logger.getLogger(Tester.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        });
    }
    
    static void assessGUI() //Present the current question and command to the student
    {
        sentenceField.setText(assessment.get(currentQuestion).getSentence());
        descriptionField.setText(assessment.get(currentQuestion).getCommand());
        answerField.setText("");
    }
    
    void answerQuestion(String answer) //Grade the student's response and proceed to the next question
    {
        System.out.println("Current grade: "+grade);
        answers.add(answer); //store the answer
        mark(answer, assessment.get(currentQuestion)); //grade the answer
        System.out.println("Correct Answer: "+assessment.get(currentQuestion).getAnswer());
        System.out.println("New grade: "+grade+"\n");
        answered.add(currentQuestion); //store the question as answered
        nextQuestion(); //proceed to the next question
    }
    
    void nextQuestion()
    {
        if (currentQuestion<assessment.size()-1) //if the exercise is not complete, present the next question
        {
            currentQuestion++;
            assessGUI();
        }
        else if (currentQuestion==assessment.size()-1) endAssessment(); //exercise complete, end the assessment session
    }
    
    void endAssessment() //end the session and transition into feedback window
    {
        System.out.println("Final Grade: "+grade);
        this.setVisible(false);
        new Feedback().setVisible(true);
    }
    
    static void initialise() throws FileNotFoundException
    {   
        assessmentSize = Configurator.assessmentSize; //retrieve desired assessment size from Configurator
        bloomKeywords = new KeywordList("keywords.txt"); //read in the action keywords for Bloom's classification
        
        //DemoDriver.GeneratorMethod(Configurator.scramble, Configurator.sentencePlural, Configurator.verbPlural, Configurator.nounPlural);     
        /*Use above line, with Nikhil Gilbert's classes in the same folder as these classes, 
        in order to use an entirely newly generated list of questions for each run. Will be shown
        during final demonstration. Note that chosen question types on Configuration page are sent to 
        his Generator. Hence, his generator is responsible for filtering the desired question types, not this component.*/
        
        loadQuestions("questions.txt"); //read in the questions from the question pool
        roots = new ArrayList();
        Scanner sc = new Scanner(new File("roots.txt"));
        while (sc.hasNext())
        {
            roots.add(sc.nextLine()); //read in the verb roots from the verb root text file
        }
        
        assignWeighting(); //assign the weighting for the levels of blooms taxonomy using keywords
        classify(); //classify questions based on weighting
        System.out.println("Weight values for questions and corresponding level:\n");
        for (Question q : questions) //print each question, its weighting and corresponding level
        {
            System.out.println(q.getCommand()+"\t"+q.weights[0]+"/"+q.weights[1]+"/"+q.weights[2]+"/"+q.weights[3]+"/"+q.weights[4]+"/"+q.weights[5]+"\t\t"+q.level); 
        }
        System.out.println("");
        evaluateDifficulty(); //evaluate the complexity measure of each question
        System.out.println("");
        
        Collections.sort(questions, new Comparator<Question>() { //sort the questions by complexity in descending order
        @Override
        public int compare(Question o1, Question o2) {
            if (o1.difficulty<o2.difficulty) return 1;
        
            else if (o1.difficulty==o2.difficulty) return 0;
        
            else return -1;}
        });
        
        for (Question q : questions) //print each question and its complexity measure
        {
            System.out.println("Type: "+q.type+", "+q.getSentence()+", Complexity Measure is: "+q.difficulty);
        }
    }
    
    static void loadQuestions(String filename) throws FileNotFoundException //read in the questions from the question pool
    {
        Scanner sc = new Scanner(new File(filename));
        while (sc.hasNextLine())
        {
            String temp = sc.nextLine();
            questions.add(new Question(temp));
        }
        System.out.println("Questions loaded.\n");
    }
    
    static void assignWeighting() //use the Bloom's action keywords list to weight questions accordingly
    {
        for (int i = 0; i < questions.size(); i++) {
            for (Word commandWord : questions.get(i).commandWords) {
                if (commandWord.type.equals("v") && bloomKeywords.hasKeyword(commandWord.text.toLowerCase(), "Knowledge")) {
                    questions.get(i).weights[0] += bloomKeywords.getWeight(commandWord.text.toLowerCase(), "Knowledge");
                }
                if (commandWord.type.equals("v") && bloomKeywords.hasKeyword(commandWord.text.toLowerCase(), "Comprehension")) {
                    questions.get(i).weights[1] += bloomKeywords.getWeight(commandWord.text.toLowerCase(), "Comprehension");
                }
                if (commandWord.type.equals("v") && bloomKeywords.hasKeyword(commandWord.text.toLowerCase(), "Application")) {
                    questions.get(i).weights[2] += bloomKeywords.getWeight(commandWord.text.toLowerCase(), "Application");
                }
                if (commandWord.type.equals("v") && bloomKeywords.hasKeyword(commandWord.text.toLowerCase(), "Analysis")) {
                    questions.get(i).weights[3] += bloomKeywords.getWeight(commandWord.text.toLowerCase(), "Analysis");
                }
                if (commandWord.type.equals("v") && bloomKeywords.hasKeyword(commandWord.text.toLowerCase(), "Synthesis")) {
                    questions.get(i).weights[4] += bloomKeywords.getWeight(commandWord.text.toLowerCase(), "Synthesis");
                }
                if (commandWord.type.equals("v") && bloomKeywords.hasKeyword(commandWord.text.toLowerCase(), "Evaluation")) {
                    questions.get(i).weights[5] += bloomKeywords.getWeight(commandWord.text.toLowerCase(), "Evaluation");
                }
            }
        }
        System.out.println("Weights calculated.\n");
    }
    
    static void classify() //classify questions into levels of the taxonomy based on weighting
    {
        for (int i = 0; i < questions.size(); i++) {
            questions.get(i).level = questions.get(i).getLevel();
        }
        System.out.println("Questions classified.\n");
    }
    
    static void evaluateDifficulty() throws FileNotFoundException //evaluate the complexity measure for each question
    {
        for (int i = 0; i < questions.size(); i++) {
            questions.get(i).difficulty = DifficultyEvaluator.getDifficulty(questions.get(i));
        }
        System.out.println("Complexity Measures Evaluated:");
    }
    
    static void compose() //request composition of an assessment
    {     
        newAssessment(); //clear and initialize all necessary variables
        System.out.println("\nBloom's assessment being composed.\n");
        bloomComposition(); //compose the assessment
        Collections.shuffle(assessment); //shuffle the order
        System.out.println("\nAssessment composed, size: "+assessment.size()+", average complexity: "+totalComplexity/assessment.size()+"\n");
    }
    
    static void bloomComposition() //compose an assessment with respect to Bloom's taxonomy
    {
        int kRatio = Configurator.kR;
        int cRatio = Configurator.cR;
        int apRatio = Configurator.apR;     //Getting desired ratios from Configurator
        int anRatio = Configurator.anR;
        int sRatio = Configurator.sR;
        int eRatio = Configurator.eR;
        
        int nK = (int) Math.round((kRatio/100.0) * assessmentSize);
        int nC = (int) Math.round((cRatio/100.0) * assessmentSize);
        int nAP = (int) Math.round((apRatio/100.0) * assessmentSize);   //Calculating appropriate number of questions per level
        int nAN = (int) Math.round((anRatio/100.0) * assessmentSize);   //based on assessment size and desired ratio
        int nS = (int) Math.round((sRatio/100.0) * assessmentSize);
        int nE = (int) Math.round((eRatio/100.0) * assessmentSize);
        
        //printing the number of appropriate questions per level and the number of questions available per level
        System.out.println("Appropriate number of questions per level:\n"+nK+" "+nC+" "+nAP+" "+nAN+" "+nS+" "+nE);
        System.out.println("Number of questions available per level:\nK:"+getNumInLevel("Knowledge")+", C:"+getNumInLevel("Comprehension")
        +", Ap:"+getNumInLevel("Application")+", An:"+getNumInLevel("Analysis")
        +", S:"+getNumInLevel("Synthesis")+", E:"+getNumInLevel("Evaluation")+"\n");
        
        int counter = 0;
        
        //Adding necessary questions per level (first iteration - top downwards)
        counter = addLevel("Evaluation", counter, nE);
        if (counter<nE) nS+=nE-counter;
        counter = 0;
        
        counter = addLevel("Synthesis", counter, nS);
        if (counter<nS) nAN+= nS-counter;
        counter=0;
        
        counter = addLevel("Analysis", counter, nAN);
        if (counter<nAN) nAP+= nAN-counter;
        counter=0;
        
        counter = addLevel("Application", counter, nAP);
        if (counter<nAP) nC+= nAP-counter;
        counter=0;
        
        counter = addLevel("Comprehension", counter, nC);
        if (counter<nC) nK+= nC-counter;
        counter=0;
        
        addLevel("Knowledge", counter, nK);
        
        //adding more questions if needed (second iteration - bottom upwards)
        if (assessment.size()<assessmentSize)
        {
            System.out.println("Short by: "+ (assessmentSize-assessment.size()));
            int count = assessmentSize-assessment.size();
            int temp = 0;
            
            temp = addLevel("Knowledge", temp, count);
            count-=temp;
            temp=0;

            temp = addLevel("Comprehension", temp, count);
            count-=temp;
            temp=0;

            temp = addLevel("Application", temp, count);
            count-=temp;
            temp=0;

            temp = addLevel("Analysis", temp, count);
            count-=temp;
            temp=0;

            temp = addLevel("Synthesis", temp, count);
            count-=temp;
            temp=0;

            addLevel("Evaluation", temp, count);
        }
        
        //final additions if necessary
        if (assessment.size()<assessmentSize)
        {
            System.out.println("Still short.");
            for (int i = 0; i < questions.size(); i++) {
                if (!chosen.contains(i) && assessment.size()<assessmentSize)
                {
                    assessment.add(questions.get(i));
                    chosen.add(i);
                    totalComplexity+=questions.get(i).difficulty;
                }
            }
        }
    }
    
    static int addLevel(String level, int counter, int n) //add questions from a specific level to the exercise
    {
        for (int i = 0; i < questions.size(); i++) { //only add if correct level, has not been chosen already and more questions are required
            if (questions.get(i).level.equals(level) && counter<n && !chosen.contains(i) && assessment.size()<assessmentSize){
                assessment.add(questions.get(i));
                chosen.add(i);
                totalComplexity+=questions.get(i).difficulty;
                counter++;
            }
        }
        System.out.println(counter+" items added from "+level);
        return counter;
    }
    
    /*FOR EXPERIMENT USE ONLY 
    - used to compare against Bloom's composition. 

    static void randomComposition() 
    {
        for (int i = 0; i < assessmentSize; i++) {
            assessment.add(questions.get(i));
            chosen.add(questions.get(i));
            randomdiff+=questions.get(i).difficulty;
        }
    }*/
    
    static void mark(String input, Question q) //request grading of an answer
    {
        grade += gradeAnswer(input,q);
    }
    
    static double gradeAnswer(String input, Question q) //grade an answer
    {
        if (input.split(" ").length!=q.answerWords.length){ //if answer length and correct answer length not equal then incorrect
            scores.add(0);
            System.out.println("Incorrect!");
            return 0;
        }
        
        else
        {
            if (q.getAnswer().toLowerCase().trim().equals(input.toLowerCase().trim())) { //if correct answer and user answer identical than correct
                scores.add(1);
                System.out.println("Correct!");
                return 1;
            }
            else { //else incorrect
                scores.add(0);
                System.out.println("Incorrect!");
                return 0;
            }
        }
    }
    
    static void newAssessment() //clear and initialize necessary variables for new assessment
    {
        assessment.clear();
        grade = 0;
        answered.clear();
        chosen.clear();
        currentQuestion = 0;
        questionCount = 0;
        totalComplexity=0;
    }
    
    static int getNumInLevel(String level) //return number of questions available for a given level
    {
        int count = 0;
        for (Question q: questions)
        {
            if (q.level.equalsIgnoreCase(level)) count ++;
        }
        return count;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    public static javax.swing.JTextField answerField;
    public static javax.swing.JTextField descriptionField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    public static javax.swing.JTextField sentenceField;
    private static javax.swing.JButton submitButton;
    // End of variables declaration//GEN-END:variables
}
